@@ -14,10 +14,16 @@
 networking and storage of containers, and compute hosts on which they run)."""
 
 from zun.common import consts
+from zun.common import exception
 from zun.common import profiler
+from zun.compute import container_actions
 from zun.compute import rpcapi
+import zun.conf
+from zun import objects
 from zun.scheduler import client as scheduler_client
 
+CONF = zun.conf.CONF
+
 
 @profiler.trace_cls("rpc")
 class API(object):
@@ -28,8 +34,13 @@ class API(object):
         self.scheduler_client = scheduler_client.SchedulerClient()
         super(API, self).__init__()
 
+    def _record_action_start(self, context, container, action):
+        objects.ContainerAction.action_start(context, container.uuid,
+                                             action, want_result=False)
+
     def container_create(self, context, new_container, extra_spec,
-                         requested_networks):
+                         requested_networks, requested_volumes, run,
+                         pci_requests=None):
         host_state = None
         try:
             host_state = self._schedule_container(context, new_container,
@@ -40,24 +51,22 @@ class API(object):
             new_container.save(context)
             return
 
+        # NOTE(mkrai): Intent here is to check the existence of image
+        # before proceeding to create container. If image is not found,
+        # container create will fail with 400 status.
+        if CONF.api.enable_image_validation:
+            images = self.rpcapi.image_search(
+                context, new_container.image,
+                new_container.image_driver, True, host_state['host'])
+            if not images:
+                raise exception.ImageNotFound(image=new_container.image)
+
+        self._record_action_start(context, new_container,
+                                  container_actions.CREATE)
         self.rpcapi.container_create(context, host_state['host'],
                                      new_container, host_state['limits'],
-                                     requested_networks)
-
-    def container_run(self, context, new_container, extra_spec,
-                      requested_networks):
-        host_state = None
-        try:
-            host_state = self._schedule_container(context, new_container,
-                                                  extra_spec)
-        except Exception as exc:
-            new_container.status = consts.ERROR
-            new_container.status_reason = str(exc)
-            new_container.save(context)
-            return
-
-        self.rpcapi.container_run(context, host_state['host'], new_container,
-                                  host_state['limits'], requested_networks)
+                                     requested_networks, requested_volumes,
+                                     run, pci_requests)
 
     def _schedule_container(self, context, new_container, extra_spec):
         dests = self.scheduler_client.select_destinations(context,
@@ -66,24 +75,31 @@ class API(object):
         return dests[0]
 
     def container_delete(self, context, container, *args):
+        self._record_action_start(context, container, container_actions.DELETE)
         return self.rpcapi.container_delete(context, container, *args)
 
     def container_show(self, context, container, *args):
         return self.rpcapi.container_show(context, container, *args)
 
     def container_reboot(self, context, container, *args):
+        self._record_action_start(context, container, container_actions.REBOOT)
         return self.rpcapi.container_reboot(context, container, *args)
 
     def container_stop(self, context, container, *args):
+        self._record_action_start(context, container, container_actions.STOP)
         return self.rpcapi.container_stop(context, container, *args)
 
     def container_start(self, context, container):
+        self._record_action_start(context, container, container_actions.START)
         return self.rpcapi.container_start(context, container)
 
     def container_pause(self, context, container):
+        self._record_action_start(context, container, container_actions.PAUSE)
         return self.rpcapi.container_pause(context, container)
 
     def container_unpause(self, context, container):
+        self._record_action_start(context, container,
+                                  container_actions.UNPAUSE)
         return self.rpcapi.container_unpause(context, container)
 
     def container_logs(self, context, container, stdout, stderr,
@@ -98,6 +114,7 @@ class API(object):
         return self.rpcapi.container_exec_resize(context, container, *args)
 
     def container_kill(self, context, container, *args):
+        self._record_action_start(context, container, container_actions.KILL)
         return self.rpcapi.container_kill(context, container, *args)
 
     def container_update(self, context, container, *args):
@@ -116,8 +133,15 @@ class API(object):
         return self.rpcapi.container_get_archive(context, container, *args)
 
     def add_security_group(self, context, container, *args):
+        self._record_action_start(context, container,
+                                  container_actions.ADD_SECURITY_GROUP)
         return self.rpcapi.add_security_group(context, container, *args)
 
+    def remove_security_group(self, context, container, *args):
+        self._record_action_start(context, container,
+                                  container_actions.REMOVE_SECURITY_GROUP)
+        return self.rpcapi.remove_security_group(context, container, *args)
+
     def container_put_archive(self, context, container, *args):
         return self.rpcapi.container_put_archive(context, container, *args)
 
@@ -125,16 +149,18 @@ class API(object):
         return self.rpcapi.container_stats(context, container)
 
     def container_commit(self, context, container, *args):
+        self._record_action_start(context, container, container_actions.COMMIT)
         return self.rpcapi.container_commit(context, container, *args)
 
     def image_pull(self, context, image):
         return self.rpcapi.image_pull(context, image)
 
-    def image_search(self, context, image, image_driver, *args):
-        return self.rpcapi.image_search(context, image, image_driver, *args)
+    def image_search(self, context, image, image_driver, exact_match, *args):
+        return self.rpcapi.image_search(context, image, image_driver,
+                                        exact_match, *args)
 
-    def capsule_create(self, context, new_capsule,
-                       requested_networks=None, extra_spec=None):
+    def capsule_create(self, context, new_capsule, requested_networks=None,
+                       requested_volumes=None, extra_spec=None):
         host_state = None
         try:
             host_state = self._schedule_container(context, new_capsule,
@@ -145,4 +171,18 @@ class API(object):
             new_capsule.save(context)
             return
         self.rpcapi.capsule_create(context, host_state['host'], new_capsule,
-                                   requested_networks, host_state['limits'])
+                                   requested_networks, requested_volumes,
+                                   host_state['limits'])
+
+    def capsule_delete(self, context, capsule, *args):
+        return self.rpcapi.capsule_delete(context, capsule, *args)
+
+    def network_detach(self, context, container, *args):
+        self._record_action_start(context, container,
+                                  container_actions.NETWORK_DETACH)
+        return self.rpcapi.network_detach(context, container, *args)
+
+    def network_attach(self, context, container, *args):
+        self._record_action_start(context, container,
+                                  container_actions.NETWORK_ATTACH)
+        return self.rpcapi.network_attach(context, container, *args)
