@@ -56,18 +56,18 @@ class API(rpc_service.API):
             transport, context, topic=zun.conf.CONF.compute.topic)
 
     def container_create(self, context, host, container, limits,
-                         requested_networks):
+                         requested_networks, requested_volumes, run,
+                         pci_requests):
         self._cast(host, 'container_create', limits=limits,
-                   requested_networks=requested_networks, container=container)
-
-    def container_run(self, context, host, container, limits,
-                      requested_networks):
-        self._cast(host, 'container_run', limits=limits,
-                   requested_networks=requested_networks, container=container)
+                   requested_networks=requested_networks,
+                   requested_volumes=requested_volumes,
+                   container=container,
+                   run=run,
+                   pci_requests=pci_requests)
 
     @check_container_host
     def container_delete(self, context, container, force):
-        return self._call(container.host, 'container_delete',
+        return self._cast(container.host, 'container_delete',
                           container=container, force=force)
 
     @check_container_host
@@ -159,6 +159,10 @@ class API(rpc_service.API):
         return self._cast(container.host, 'add_security_group',
                           container=container, security_group=security_group)
 
+    def remove_security_group(self, context, container, security_group):
+        return self._cast(container.host, 'remove_security_group',
+                          container=container, security_group=security_group)
+
     def image_pull(self, context, image):
         # NOTE(hongbin): Image API doesn't support multiple compute nodes
         # scenario yet, so we temporarily set host to None and rpc will
@@ -166,18 +170,28 @@ class API(rpc_service.API):
         host = None
         self._cast(host, 'image_pull', image=image)
 
-    def image_search(self, context, image, image_driver, exact_match):
-        # NOTE(hongbin): Image API doesn't support multiple compute nodes
-        # scenario yet, so we temporarily set host to None and rpc will
-        # choose an arbitrary host.
-        host = None
+    def image_search(self, context, image, image_driver, exact_match,
+                     host=None):
         return self._call(host, 'image_search', image=image,
                           image_driver_name=image_driver,
                           exact_match=exact_match)
 
     def capsule_create(self, context, host, capsule,
-                       requested_networks, limits):
+                       requested_networks, requested_volumes, limits):
         self._cast(host, 'capsule_create',
                    capsule=capsule,
                    requested_networks=requested_networks,
+                   requested_volumes=requested_volumes,
                    limits=limits)
+
+    def capsule_delete(self, context, capsule):
+        return self._call(capsule.host, 'capsule_delete',
+                          capsule=capsule)
+
+    def network_detach(self, context, container, network):
+        return self._call(container.host, 'network_detach',
+                          container=container, network=network)
+
+    def network_attach(self, context, container, network):
+        return self._call(container.host, 'network_attach',
+                          container=container, network=network)
