@@ -23,7 +23,6 @@ from oslo_utils import timeutils
 from zun.common import consts
 from zun.common import exception
 from zun.common.i18n import _
-from zun.common import nova
 from zun.common import utils
 from zun.common.utils import check_container_id
 import zun.conf
@@ -32,6 +31,7 @@ from zun.container.docker import utils as docker_utils
 from zun.container import driver
 from zun.network import network as zun_network
 from zun import objects
+from zun.volume import driver as vol_driver
 
 
 CONF = zun.conf.CONF
@@ -43,7 +43,18 @@ def is_not_found(e):
     return '404' in str(e)
 
 
+def is_conflict(e):
+    conflict_infos = ['not running', 'not paused', 'paused']
+    for info in conflict_infos:
+        if info in str(e):
+            return True
+    return False
+
+
 def handle_not_found(e, context, container, do_not_raise=False):
+    if container.status == consts.DELETING:
+        return
+
     if container.auto_remove:
         container.status = consts.DELETED
     else:
@@ -68,6 +79,8 @@ def wrap_docker_error(function):
         except exception.DockerError as e:
             if is_not_found(e):
                 handle_not_found(e, context, container)
+            if is_conflict(e):
+                raise exception.Conflict(_("%s") % str(e))
             raise
 
     return decorated_function
@@ -101,6 +114,11 @@ class DockerDriver(driver.ContainerDriver):
         with docker_utils.docker_client() as docker:
             return docker.get_image(name)
 
+    def delete_image(self, image):
+        with docker_utils.docker_client() as docker:
+            LOG.debug('Deleting image %s', image)
+            return docker.remove_image(image)
+
     def images(self, repo, quiet=False):
         with docker_utils.docker_client() as docker:
             return docker.images(repo, quiet)
@@ -113,19 +131,17 @@ class DockerDriver(driver.ContainerDriver):
             except Exception:
                 LOG.warning("Unable to read image data from tarfile")
 
-    def create(self, context, container, image, requested_networks):
+    def create(self, context, container, image, requested_networks,
+               requested_volumes):
         sandbox_id = container.get_sandbox_id()
-        network_standalone = False if sandbox_id else True
 
         with docker_utils.docker_client() as docker:
             network_api = zun_network.api(context=context, docker_api=docker)
             name = container.name
             LOG.debug('Creating container with image %(image)s name %(name)s',
                       {'image': image['image'], 'name': name})
-            if network_standalone:
-                self._provision_network(context, network_api,
-                                        requested_networks)
-
+            self._provision_network(context, network_api, requested_networks)
+            binds = self._get_binds(context, requested_volumes)
             kwargs = {
                 'name': self.get_container_name(container),
                 'command': container.command,
@@ -135,15 +151,28 @@ class DockerDriver(driver.ContainerDriver):
                 'tty': container.interactive,
                 'stdin_open': container.interactive,
             }
+            if not sandbox_id:
+                # Sandbox is not used so it is legitimate to customize
+                # the container's hostname
+                kwargs['hostname'] = container.hostname
+
+            runtime = container.runtime if container.runtime\
+                else CONF.container_runtime
 
             host_config = {}
+            host_config['runtime'] = runtime
+            host_config['binds'] = binds
+            kwargs['volumes'] = [b['bind'] for b in binds.values()]
             if sandbox_id:
                 host_config['network_mode'] = 'container:%s' % sandbox_id
                 # TODO(hongbin): Uncomment this after docker-py add support for
                 # container mode for pid namespace.
                 # host_config['pid_mode'] = 'container:%s' % sandbox_id
                 host_config['ipc_mode'] = 'container:%s' % sandbox_id
-                host_config['volumes_from'] = sandbox_id
+            else:
+                self._process_networking_config(
+                    context, container, requested_networks, host_config,
+                    kwargs, docker)
             if container.auto_remove:
                 host_config['auto_remove'] = container.auto_remove
             if container.memory is not None:
@@ -156,36 +185,82 @@ class DockerDriver(driver.ContainerDriver):
                 name = container.restart_policy['Name']
                 host_config['restart_policy'] = {'Name': name,
                                                  'MaximumRetryCount': count}
+            if container.disk:
+                disk_size = str(container.disk) + 'G'
+                host_config['storage_opt'] = {'size': disk_size}
+
             kwargs['host_config'] = docker.create_host_config(**host_config)
             image_repo = image['repo'] + ":" + image['tag']
             response = docker.create_container(image_repo, **kwargs)
             container.container_id = response['Id']
 
-            if network_standalone:
-                addresses = self._setup_network_for_container(
-                    context, container, requested_networks, network_api)
-                container.addresses = addresses
+            addresses = self._setup_network_for_container(
+                context, container, requested_networks, network_api)
+            container.addresses = addresses
 
-            container.status = consts.CREATED
-            container.status_reason = None
+            response = docker.inspect_container(container.container_id)
+            self._populate_container(container, response)
             container.save(context)
             return container
 
+    def _process_networking_config(self, context, container,
+                                   requested_networks, host_config,
+                                   container_kwargs, docker):
+        network_api = zun_network.api(context=context, docker_api=docker)
+        # Process the first requested network at create time. The rest
+        # will be processed after create.
+        requested_network = requested_networks.pop()
+        docker_net_name = self._get_docker_network_name(
+            context, requested_network['network'])
+        security_group_ids = utils.get_security_group_ids(
+            context, container.security_groups)
+        addresses, port = network_api.create_or_update_port(
+            container, docker_net_name, requested_network, security_group_ids)
+        container.addresses = {requested_network['network']: addresses}
+
+        ipv4_address = None
+        ipv6_address = None
+        for address in addresses:
+            if address['version'] == 4:
+                ipv4_address = address['addr']
+            if address['version'] == 6:
+                ipv6_address = address['addr']
+
+        endpoint_config = docker.create_endpoint_config(
+            ipv4_address=ipv4_address, ipv6_address=ipv6_address)
+        network_config = docker.create_networking_config({
+            docker_net_name: endpoint_config})
+
+        host_config['network_mode'] = docker_net_name
+        container_kwargs['networking_config'] = network_config
+        container_kwargs['mac_address'] = port['mac_address']
+
     def _provision_network(self, context, network_api, requested_networks):
         for rq_network in requested_networks:
             self._get_or_create_docker_network(
                 context, network_api, rq_network['network'])
 
+    def _get_binds(self, context, requested_volumes):
+        binds = {}
+        for volume in requested_volumes:
+            volume_driver = vol_driver.driver(provider=volume.volume_provider,
+                                              context=context)
+            source, destination = volume_driver.bind_mount(volume)
+            binds[source] = {'bind': destination}
+        return binds
+
     def _setup_network_for_container(self, context, container,
                                      requested_networks, network_api):
-        security_group_ids = utils.get_security_group_ids(context, container.
-                                                          security_groups)
-        # Container connects to the bridge network by default so disconnect
-        # the container from it before connecting it to neutron network.
-        # This avoids potential conflict between these two networks.
-        network_api.disconnect_container_from_network(container, 'bridge')
+        security_group_ids = utils.get_security_group_ids(
+            context, container.security_groups)
         addresses = {}
+        if container.addresses:
+            addresses = container.addresses
         for network in requested_networks:
+            if network['network'] in addresses:
+                # This network is already setup so skip it
+                continue
+
             docker_net_name = self._get_docker_network_name(
                 context, network['network'])
             addrs = network_api.connect_container_to_network(
@@ -204,8 +279,7 @@ class DockerDriver(driver.ContainerDriver):
             if teardown_network:
                 network_api = zun_network.api(context=context,
                                               docker_api=docker)
-                self._cleanup_network_for_container(context, container,
-                                                    network_api)
+                self._cleanup_network_for_container(container, network_api)
 
             if container.container_id:
                 try:
@@ -216,11 +290,12 @@ class DockerDriver(driver.ContainerDriver):
                         return
                     raise
 
-    def _cleanup_network_for_container(self, context, container, network_api):
+    @wrap_docker_error
+    def _cleanup_network_for_container(self, container, network_api):
         if not container.addresses:
             return
         for neutron_net in container.addresses:
-            docker_net = self._get_docker_network_name(context, neutron_net)
+            docker_net = neutron_net
             network_api.disconnect_container_from_network(
                 container, docker_net, neutron_network_id=neutron_net)
 
@@ -232,20 +307,21 @@ class DockerDriver(driver.ContainerDriver):
 
         db_containers = objects.Container.list_by_host(context, CONF.host)
         for db_container in db_containers:
-            if db_container.status in (consts.CREATING, consts.DELETED):
+            if db_container.status in (consts.CREATING, consts.DELETING,
+                                       consts.DELETED):
                 # Skip populating db record since the container is in a
                 # unstable state.
                 continue
 
             container_id = db_container.container_id
             docker_container = id_to_container_map.get(container_id)
-            if not docker_container:
+            if not container_id or not docker_container:
                 if db_container.auto_remove:
                     db_container.status = consts.DELETED
                     db_container.save(context)
                 else:
-                    LOG.warning("Container was recorded in DB but missing in "
-                                "docker")
+                    LOG.warning("Container %s was recorded in DB but missing "
+                                "in docker", db_container.uuid)
                 continue
 
             self._populate_container(db_container, docker_container)
@@ -258,9 +334,11 @@ class DockerDriver(driver.ContainerDriver):
             return
 
         id_to_db_container_map = {container.container_id: container
-                                  for container in db_containers}
+                                  for container in db_containers
+                                  if container.container_id}
         id_to_container_map = {container.container_id: container
-                               for container in containers}
+                               for container in containers
+                               if container.container_id}
 
         for cid in (six.viewkeys(id_to_container_map) &
                     six.viewkeys(id_to_db_container_map)):
@@ -316,8 +394,8 @@ class DockerDriver(driver.ContainerDriver):
         delta = timeutils.utcnow() - st
         time_dict = {}
         time_dict['days'] = delta.days
-        time_dict['hours'] = delta.seconds//3600
-        time_dict['minutes'] = (delta.seconds % 3600)//60
+        time_dict['hours'] = delta.seconds // 3600
+        time_dict['minutes'] = (delta.seconds % 3600) // 60
         time_dict['seconds'] = delta.seconds
         if time_dict['days']:
             return '{} days'.format(time_dict['days'])
@@ -331,7 +409,22 @@ class DockerDriver(driver.ContainerDriver):
 
     def _populate_container(self, container, response):
         state = response.get('State')
-        if type(state) is dict:
+        self._populate_container_state(container, state)
+
+        config = response.get('Config')
+        if config:
+            self._populate_hostname_and_ports(container, config)
+            self._populate_command(container, config)
+
+        hostconfig = response.get('HostConfig')
+        if hostconfig:
+            container.runtime = hostconfig.get('Runtime')
+
+    def _populate_container_state(self, container, state):
+        if not state:
+            container.status = consts.UNKNOWN
+            container.status_detail = None
+        elif type(state) is dict:
             status_detail = ''
             if state.get('Error'):
                 container.status = consts.ERROR
@@ -355,47 +448,51 @@ class DockerDriver(driver.ContainerDriver):
                 started_at = self.format_status_detail(state.get('StartedAt'))
                 finished_at = self.format_status_detail(
                     state.get('FinishedAt'))
-                if started_at == "":
+                if started_at == "" and container.status == consts.CREATING:
                     container.status = consts.CREATED
                     container.status_detail = "Created"
-                elif finished_at == "":
+                elif (started_at == "" and
+                        container.status in (consts.CREATED, consts.ERROR)):
+                    pass
+                elif started_at != "" and finished_at == "":
                     container.status = consts.UNKNOWN
                     container.status_detail = ""
-                else:
+                elif started_at != "" and finished_at != "":
                     container.status = consts.STOPPED
                     container.status_detail = "Exited({}) {} ago ".format(
                         state.get('ExitCode'), finished_at)
             if status_detail is None:
                 container.status_detail = None
         else:
-            if state.lower() == 'created':
+            state = state.lower()
+            if state == 'created' and container.status == consts.CREATING:
                 container.status = consts.CREATED
-            elif state.lower() == 'paused':
+            elif (state == 'created' and
+                    container.status in (consts.CREATED, consts.ERROR)):
+                pass
+            elif state == 'ERROR':
                 container.status = consts.PAUSED
-            elif state.lower() == 'running':
+            elif state == 'running':
                 container.status = consts.RUNNING
-            elif state.lower() == 'dead':
+            elif state == 'dead':
                 container.status = consts.ERROR
-            elif state.lower() in ('restarting', 'exited', 'removing'):
+            elif state in ('restarting', 'exited', 'removing'):
                 container.status = consts.STOPPED
             else:
                 container.status = consts.UNKNOWN
             container.status_detail = None
 
-        config = response.get('Config')
-        if config:
-            self._populate_hostname_and_ports(container, config)
-            self._populate_command(container, config)
-
     def _populate_command(self, container, config):
         command_list = config.get('Cmd')
-        command_str = ' '.join(command_list)
-        if command_str:
-            container.command = command_str
+        command_str = None
+        if command_list:
+            command_str = ' '.join(command_list)
+        container.command = command_str
 
     def _populate_hostname_and_ports(self, container, config):
-        # populate hostname
-        container.hostname = config.get('Hostname')
+        # populate hostname only when container.hostname wasn't set
+        if container.hostname is None:
+            container.hostname = config.get('Hostname')
         # populate ports
         ports = []
         exposed_ports = config.get('ExposedPorts')
@@ -503,7 +600,10 @@ class DockerDriver(driver.ContainerDriver):
                 raise exception.Conflict(_(
                     "Timeout on executing command: %s") % command)
             inspect_res = docker.exec_inspect(exec_id)
-            return {"output": output, "exit_code": inspect_res['ExitCode']}
+            return {"output": output,
+                    "exit_code": inspect_res['ExitCode'],
+                    "exec_id": None,
+                    "url": None}
 
     def execute_resize(self, exec_id, height, width):
         height = int(height)
@@ -525,6 +625,8 @@ class DockerDriver(driver.ContainerDriver):
                 docker.kill(container.container_id)
             else:
                 docker.kill(container.container_id, signal)
+            container.status = consts.STOPPED
+            container.status_reason = None
             return container
 
     @check_container_id
@@ -576,15 +678,26 @@ class DockerDriver(driver.ContainerDriver):
     @wrap_docker_error
     def get_archive(self, context, container, path):
         with docker_utils.docker_client() as docker:
-            stream, stat = docker.get_archive(container.container_id, path)
-            filedata = stream.read()
-            return filedata, stat
+            try:
+                stream, stat = docker.get_archive(
+                    container.container_id, path)
+                filedata = stream.read()
+                return filedata, stat
+            except errors.APIError as api_error:
+                if is_not_found(api_error):
+                    raise exception.Invalid(_("%s") % str(api_error))
+                raise
 
     @check_container_id
     @wrap_docker_error
     def put_archive(self, context, container, path, data):
         with docker_utils.docker_client() as docker:
-            docker.put_archive(container.container_id, path, data)
+            try:
+                docker.put_archive(container.container_id, path, data)
+            except errors.APIError as api_error:
+                if is_not_found(api_error):
+                    raise exception.Invalid(_("%s") % str(api_error))
+                raise
 
     @check_container_id
     @wrap_docker_error
@@ -642,13 +755,25 @@ class DockerDriver(driver.ContainerDriver):
         return value.encode('utf-8')
 
     def create_sandbox(self, context, container, requested_networks,
+                       requested_volumes,
                        image='kubernetes/pause'):
         with docker_utils.docker_client() as docker:
             network_api = zun_network.api(context=context, docker_api=docker)
             self._provision_network(context, network_api, requested_networks)
+            binds = self._get_binds(context, requested_volumes)
+            host_config = {'binds': binds}
             name = self.get_sandbox_name(container)
-            sandbox = docker.create_container(image, name=name,
-                                              hostname=name[:63])
+            volumes = [b['bind'] for b in binds.values()]
+            kwargs = {
+                'name': name,
+                'hostname': name[:63],
+                'volumes': volumes,
+            }
+            self._process_networking_config(
+                context, container, requested_networks, host_config,
+                kwargs, docker)
+            kwargs['host_config'] = docker.create_host_config(**host_config)
+            sandbox = docker.create_container(image, **kwargs)
             container.set_sandbox_id(sandbox['Id'])
             addresses = self._setup_network_for_container(
                 context, container, requested_networks, network_api)
@@ -662,30 +787,50 @@ class DockerDriver(driver.ContainerDriver):
             docker.start(sandbox['Id'])
             return sandbox['Id']
 
+    def attach_volume(self, context, volume_mapping):
+        volume_driver = vol_driver.driver(
+            provider=volume_mapping.volume_provider,
+            context=context)
+        volume_driver.attach(volume_mapping)
+
+    def detach_volume(self, context, volume_mapping):
+        volume_driver = vol_driver.driver(
+            provider=volume_mapping.volume_provider,
+            context=context)
+        volume_driver.detach(volume_mapping)
+
+    def delete_volume(self, context, volume_mapping):
+        volume_driver = vol_driver.driver(
+            provider=volume_mapping.volume_provider,
+            context=context)
+        volume_driver.delete(volume_mapping)
+
+    def is_volume_available(self, context, volume_mapping):
+        volume_driver = vol_driver.driver(
+            provider=volume_mapping.volume_provider,
+            context=context)
+        return volume_driver.is_volume_available(volume_mapping)
+
     def _get_or_create_docker_network(self, context, network_api,
                                       neutron_net_id):
-        # Append project_id to the network name to avoid name collision
-        # across projects.
         docker_net_name = self._get_docker_network_name(context,
                                                         neutron_net_id)
         docker_networks = network_api.list_networks(names=[docker_net_name])
         if not docker_networks:
             network_api.create_network(neutron_net_id=neutron_net_id,
                                        name=docker_net_name)
-            docker_networks = network_api.list_networks(
-                names=[docker_net_name])
 
     def _get_docker_network_name(self, context, neutron_net_id):
-        # Append project_id to the network name to avoid name collision
-        # across projects.
-        return neutron_net_id + '-' + context.project_id
+        # Note(kiseok7): neutron_net_id is a unique ID in neutron networks and
+        # docker networks.
+        # so it will not be duplicated across projects.
+        return neutron_net_id
 
     def delete_sandbox(self, context, container):
         sandbox_id = container.get_sandbox_id()
         with docker_utils.docker_client() as docker:
             network_api = zun_network.api(context=context, docker_api=docker)
-            self._cleanup_network_for_container(context, container,
-                                                network_api)
+            self._cleanup_network_for_container(container, network_api)
             try:
                 docker.remove_container(sandbox_id, force=True)
             except errors.APIError as api_error:
@@ -752,119 +897,65 @@ class DockerDriver(driver.ContainerDriver):
             network_api.add_security_groups_to_ports(container,
                                                      [security_group])
 
+    def remove_security_group(self, context, container, security_group):
+
+        with docker_utils.docker_client() as docker:
+            network_api = zun_network.api(context=context,
+                                          docker_api=docker)
+            network_api.remove_security_groups_from_ports(container,
+                                                          [security_group])
+
     def get_available_nodes(self):
         return [self._host.get_hostname()]
 
-
-class NovaDockerDriver(DockerDriver):
-    capabilities = {
-        "support_sandbox": True,
-        "support_standalone": False,
-    }
-
-    def add_security_group(self, context, container, security_group, **kwargs):
-        msg = "NovaDockerDriver does not support security_groups"
-        raise exception.ZunException(msg)
-
-    def create_sandbox(self, context, container, key_name=None,
-                       flavor='m1.tiny', image='kubernetes/pause',
-                       nics='auto'):
-        # FIXME(hongbin): We elevate to admin privilege because the default
-        # policy in nova disallows non-admin users to create instance in
-        # specified host. This is not ideal because all nova instances will
-        # be created at service admin tenant now, which breaks the
-        # multi-tenancy model. We need to fix it.
-        elevated = context.elevated()
-        novaclient = nova.NovaClient(elevated)
-        name = self.get_sandbox_name(container)
-        if container.host != CONF.host:
-            raise exception.ZunException(_(
-                "Host mismatch: container should be created at host '%s'.") %
-                container.host)
-        # NOTE(hongbin): The format of availability zone is ZONE:HOST:NODE
-        # However, we just want to specify host, so it is ':HOST:'
-        az = ':%s:' % container.host
-        sandbox = novaclient.create_server(name=name, image=image,
-                                           flavor=flavor, key_name=key_name,
-                                           nics=nics, availability_zone=az)
-        self._ensure_active(novaclient, sandbox)
-        sandbox_id = self._find_container_by_server_name(name)
-        return sandbox_id
-
-    def _ensure_active(self, novaclient, server, timeout=300):
-        """Wait until the Nova instance to become active."""
-        def _check_active():
-            return novaclient.check_active(server)
-
-        success_msg = "Created server %s successfully." % server.id
-        timeout_msg = ("Failed to create server %s. Timeout waiting for "
-                       "server to become active.") % server.id
-        utils.poll_until(_check_active,
-                         sleep_time=CONF.default_sleep_time,
-                         time_out=timeout or CONF.default_timeout,
-                         success_msg=success_msg, timeout_msg=timeout_msg)
-
-    def delete_sandbox(self, context, sandbox_id):
-        elevated = context.elevated()
-        novaclient = nova.NovaClient(elevated)
-        server_name = self._find_server_by_container_id(sandbox_id)
-        if not server_name:
-            LOG.warning("Cannot find server name for sandbox %s", sandbox_id)
-            return
-
-        server_id = novaclient.delete_server(server_name)
-        self._ensure_deleted(novaclient, server_id)
-
-    def stop_sandbox(self, context, sandbox_id):
-        elevated = context.elevated()
-        novaclient = nova.NovaClient(elevated)
-        server_name = self._find_server_by_container_id(sandbox_id)
-        if not server_name:
-            LOG.warning("Cannot find server name for sandbox %s", sandbox_id)
-            return
-        novaclient.stop_server(server_name)
-
-    def _ensure_deleted(self, novaclient, server_id, timeout=300):
-        """Wait until the Nova instance to be deleted."""
-        def _check_delete_complete():
-            return novaclient.check_delete_server_complete(server_id)
-
-        success_msg = "Delete server %s successfully." % server_id
-        timeout_msg = ("Failed to create server %s. Timeout waiting for "
-                       "server to be deleted.") % server_id
-        utils.poll_until(_check_delete_complete,
-                         sleep_time=CONF.default_sleep_time,
-                         time_out=timeout or CONF.default_timeout,
-                         success_msg=success_msg, timeout_msg=timeout_msg)
-
-    def get_addresses(self, context, container):
-        elevated = context.elevated()
-        novaclient = nova.NovaClient(elevated)
-        sandbox_id = container.get_sandbox_id()
-        if sandbox_id:
-            server_name = self._find_server_by_container_id(sandbox_id)
-            if server_name:
-                # TODO(hongbin): Standardize the format of addresses
-                return novaclient.get_addresses(server_name)
-            else:
-                return None
-        else:
-            return None
-
-    def _find_container_by_server_name(self, name):
+    @wrap_docker_error
+    def network_detach(self, context, container, network):
         with docker_utils.docker_client() as docker:
-            for info in docker.list_instances(inspect=True):
-                if info['Config'].get('Hostname') == name:
-                    return info['Id']
-            raise exception.ZunException(_(
-                "Cannot find container with name %s") % name)
+            network_api = zun_network.api(context,
+                                          docker_api=docker)
+            docker_net = self._get_docker_network_name(context, network)
+            network_api.disconnect_container_from_network(container,
+                                                          docker_net, network)
 
-    def _find_server_by_container_id(self, container_id):
+            # Only clear network info related to this network
+            # Cannot del container.address directly which will not update
+            # changed fields of the container objects as the del operate on
+            # the addresses object, only base.getter will called.
+            update = container.addresses
+            del update[network]
+            container.addresses = update
+            container.save(context)
+
+    def network_attach(self, context, container, network):
         with docker_utils.docker_client() as docker:
-            try:
-                info = docker.inspect_container(container_id)
-                return info['Config'].get('Hostname')
-            except errors.APIError as e:
-                if e.response.status_code != 404:
-                    raise
-                return None
+            security_group_ids = None
+            if container.security_groups:
+                security_group_ids = utils.get_security_group_ids(
+                    context, container.security_groups)
+            network_api = zun_network.api(context,
+                                          docker_api=docker)
+            if network in container.addresses:
+                raise exception.ZunException('Container %(container)s has'
+                                             ' alreay connected to the network'
+                                             '%(network)s.'
+                                             % {'container': container['uuid'],
+                                                'network': network})
+            self._get_or_create_docker_network(context, network_api, network)
+            requested_network = {'network': network,
+                                 'port': '',
+                                 'v4-fixed-ip': '',
+                                 'v6-fixed-ip': '',
+                                 'preserve_on_delete': False}
+            docker_net_name = self._get_docker_network_name(context, network)
+            addrs = network_api.connect_container_to_network(
+                container, docker_net_name, requested_network,
+                security_groups=security_group_ids)
+            if addrs is None:
+                raise exception.ZunException(_(
+                    'Unexpected missing of addresses'))
+            update = {}
+            update[network] = addrs
+            addresses = container.addresses
+            addresses.update(update)
+            container.addresses = addresses
+            container.save(context)
