@@ -10,11 +10,20 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from oslo_log import log as logging
 from oslo_versionedobjects import fields
 
+from zun.common import exception
 from zun.db import api as dbapi
 from zun.objects import base
 from zun.objects import fields as z_fields
+from zun.objects import pci_device
+
+
+LOG = logging.getLogger(__name__)
+
+
+CONTAINER_OPTIONAL_ATTRS = ["pci_devices"]
 
 
 @base.ZunObjectRegistry.register
@@ -38,7 +47,13 @@ class Container(base.ZunPersistentObject, base.ZunObject):
     # Version 1.16: Add websocket_url and token
     # Version 1.17: Add security_groups
     # Version 1.18: Add auto_remove
-    VERSION = '1.18'
+    # Version 1.19: Add runtime column
+    # Version 1.20: Change runtime to String type
+    # Version 1.21: Add pci_device attribute
+    # Version 1.22: Add 'Deleting' to ContainerStatus
+    # Version 1.23: Add the missing 'pci_devices' attribute
+    # Version 1.24: Add the storage_opt attribute
+    VERSION = '1.24'
 
     fields = {
         'id': fields.IntegerField(),
@@ -71,12 +86,18 @@ class Container(base.ZunPersistentObject, base.ZunObject):
         'websocket_url': fields.StringField(nullable=True),
         'websocket_token': fields.StringField(nullable=True),
         'security_groups': fields.ListOfStringsField(nullable=True),
+        'runtime': fields.StringField(nullable=True),
+        'pci_devices': fields.ListOfObjectsField('PciDevice',
+                                                 nullable=True),
+        'disk': fields.IntegerField(nullable=True),
     }
 
     @staticmethod
     def _from_db_object(container, db_container):
         """Converts a database entity to a formal object."""
         for field in container.fields:
+            if field in ['pci_devices']:
+                continue
             setattr(container, field, db_container[field])
 
         container.obj_reset_changes()
@@ -226,3 +247,30 @@ class Container(base.ZunPersistentObject, base.ZunObject):
             self.meta = {'sandbox_id': sandbox_id}
         else:
             self.meta['sandbox_id'] = sandbox_id
+
+    def obj_load_attr(self, attrname):
+        if attrname not in CONTAINER_OPTIONAL_ATTRS:
+            raise exception.ObjectActionError(
+                action='obj_load_attr',
+                reason=_('attribute %s not lazy-loadable') % attrname)
+
+        if not self._context:
+            raise exception.OrphanedObjectError(method='obj_load_attr',
+                                                objtype=self.obj_name())
+
+        LOG.debug("Lazy-loading '%(attr)s' on %(name)s uuid %(uuid)s",
+                  {'attr': attrname,
+                   'name': self.obj_name(),
+                   'uuid': self.uuid,
+                   })
+
+        # NOTE(danms): We handle some fields differently here so that we
+        # can be more efficient
+        if attrname == 'pci_devices':
+            self._load_pci_devices()
+
+        self.obj_reset_changes([attrname])
+
+    def _load_pci_devices(self):
+        self.pci_devices = pci_device.PciDevice.list_by_container_uuid(
+            self._context, self.uuid)
