@@ -13,13 +13,11 @@
 from docker import errors
 import mock
 
-from oslo_serialization import jsonutils
 from oslo_utils import units
 
 from zun.common import consts
 from zun import conf
 from zun.container.docker.driver import DockerDriver
-from zun.container.docker.driver import NovaDockerDriver
 from zun.container.docker import utils as docker_utils
 from zun import objects
 from zun.tests.unit.container import base
@@ -67,6 +65,12 @@ class TestDockerDriver(base.DriverTestCase):
         self.driver.get_image(name='image_name')
         self.mock_docker.get_image.assert_called_once_with('image_name')
 
+    def test_delete_image(self):
+        self.mock_docker.remove_image = mock.Mock()
+        mock_image = mock.MagicMock()
+        self.driver.delete_image(mock_image)
+        self.mock_docker.remove_image.assert_called_once_with(mock_image)
+
     def test_load_image(self):
         self.mock_docker.load_image = mock.Mock()
         mock_open_file = mock.mock_open()
@@ -82,24 +86,41 @@ class TestDockerDriver(base.DriverTestCase):
 
     @mock.patch('zun.network.kuryr_network.KuryrNetwork'
                 '.connect_container_to_network')
+    @mock.patch('zun.network.kuryr_network.KuryrNetwork'
+                '.create_or_update_port')
     @mock.patch('zun.common.utils.get_security_group_ids')
     @mock.patch('zun.objects.container.Container.save')
     def test_create_image_path_is_none(self, mock_save,
                                        mock_get_security_group_ids,
+                                       mock_create_or_update_port,
                                        mock_connect):
         self.mock_docker.create_host_config = mock.Mock(
             return_value={'Id1': 'val1', 'key2': 'val2'})
         self.mock_docker.create_container = mock.Mock(
             return_value={'Id': 'val1', 'key1': 'val2'})
+        self.mock_docker.create_networking_config = mock.Mock(
+            return_value={'Id': 'val1', 'key1': 'val2'})
+        self.mock_docker.inspect_container = mock.Mock(
+            return_value={'State': 'created',
+                          'Config': {'Cmd': ['fake_command']}})
         image = {'path': '', 'image': '', 'repo': '', 'tag': ''}
         mock_container = self.mock_default_container
+        mock_container.status = 'Creating'
+        networks = [{'network': 'fake-network'}]
+        volumes = []
+        fake_port = {'mac_address': 'fake_mac'}
+        mock_create_or_update_port.return_value = ([], fake_port)
         result_container = self.driver.create(self.context, mock_container,
-                                              image, [])
+                                              image, networks, volumes)
         host_config = {}
         host_config['mem_limit'] = '512m'
         host_config['cpu_quota'] = 100000
         host_config['cpu_period'] = 100000
         host_config['restart_policy'] = {'Name': 'no', 'MaximumRetryCount': 0}
+        host_config['runtime'] = 'runc'
+        host_config['binds'] = {}
+        host_config['network_mode'] = 'fake-network'
+        host_config['storage_opt'] = {'size': '20G'}
         self.mock_docker.create_host_config.assert_called_once_with(
             **host_config)
 
@@ -112,6 +133,10 @@ class TestDockerDriver(base.DriverTestCase):
             'host_config': {'Id1': 'val1', 'key2': 'val2'},
             'stdin_open': True,
             'tty': True,
+            'hostname': 'testhost',
+            'volumes': [],
+            'networking_config': {'Id': 'val1', 'key1': 'val2'},
+            'mac_address': 'fake_mac',
         }
         self.mock_docker.create_container.assert_called_once_with(
             image['repo'] + ":" + image['tag'], **kwargs)
@@ -189,6 +214,38 @@ class TestDockerDriver(base.DriverTestCase):
             mock_container.container_id)
         self.assertEqual('fake_command', mock_container.command)
 
+    def test_show_without_command(self):
+        self.mock_docker.inspect_container = mock.Mock(
+            return_value={'State': 'running',
+                          'Config': {'Cmd': None}})
+        mock_container = mock.MagicMock()
+        self.driver.show(self.context, mock_container)
+        self.mock_docker.inspect_container.assert_called_once_with(
+            mock_container.container_id)
+        self.assertIsNone(mock_container.command)
+
+    def test_show_container_created(self):
+        self.mock_docker.inspect_container = mock.Mock(
+            return_value={'State': 'created',
+                          'Config': {'Cmd': ['fake_command']}})
+        mock_container = mock.MagicMock()
+        mock_container.status = consts.CREATING
+        self.driver.show(self.context, mock_container)
+        self.mock_docker.inspect_container.assert_called_once_with(
+            mock_container.container_id)
+        self.assertEqual(consts.CREATED, mock_container.status)
+
+    def test_show_container_create_failed(self):
+        self.mock_docker.inspect_container = mock.Mock(
+            return_value={'State': 'created',
+                          'Config': {'Cmd': ['fake_command']}})
+        mock_container = mock.MagicMock()
+        mock_container.status = consts.ERROR
+        self.driver.show(self.context, mock_container)
+        self.mock_docker.inspect_container.assert_called_once_with(
+            mock_container.container_id)
+        self.assertEqual(consts.ERROR, mock_container.status)
+
     def test_show_fail_container_id_is_none(self):
         mock_container = mock.MagicMock()
         mock_container.container_id = None
@@ -208,6 +265,19 @@ class TestDockerDriver(base.DriverTestCase):
                              consts.ERROR)
             self.assertEqual(2, mock_init.call_count)
 
+    def test_show_status_deleting(self):
+        with mock.patch.object(errors.APIError, '__str__',
+                               return_value='404 Not Found') as mock_init:
+            self.mock_docker.inspect_container = mock.Mock(
+                side_effect=errors.APIError('Error', '', ''))
+            mock_container = mock.MagicMock(status=consts.DELETING)
+            result_container = self.driver.show(self.context, mock_container)
+            self.mock_docker.inspect_container.assert_called_once_with(
+                mock_container.container_id)
+            self.assertEqual(result_container.status,
+                             consts.DELETING)
+            self.assertEqual(1, mock_init.call_count)
+
     def test_show_fail_api_error(self):
         with mock.patch.object(errors.APIError, '__str__',
                                return_value='test') as mock_init:
@@ -324,41 +394,68 @@ class TestDockerDriver(base.DriverTestCase):
 
     @mock.patch('zun.network.kuryr_network.KuryrNetwork'
                 '.connect_container_to_network')
+    @mock.patch('zun.network.kuryr_network.KuryrNetwork'
+                '.create_or_update_port')
     @mock.patch('zun.container.docker.driver.DockerDriver.get_sandbox_name')
     @mock.patch('zun.common.utils.get_security_group_ids')
     def test_create_sandbox(self, mock_get_security_group_ids,
-                            mock_get_sandbox_name, mock_connect):
+                            mock_get_sandbox_name, mock_create_or_update_port,
+                            mock_connect):
         sandbox_name = 'my_test_sandbox'
         mock_get_sandbox_name.return_value = sandbox_name
         self.mock_docker.create_container = mock.Mock(
             return_value={'Id': 'val1', 'key1': 'val2'})
+        self.mock_docker.create_networking_config = mock.Mock(
+            return_value={'Id': 'val1', 'key1': 'val2'})
+        fake_host_config = mock.Mock()
+        self.mock_docker.create_host_config.return_value = fake_host_config
         mock_container = mock.MagicMock()
-        requested_networks = []
+        hostname = 'my_hostname'
+        mock_container.hostname = hostname
+        requested_networks = [{'network': 'fake-network'}]
+        requested_volumes = []
+        fake_port = {'mac_address': 'fake_mac'}
+        mock_create_or_update_port.return_value = ([], fake_port)
         result_sandbox_id = self.driver.create_sandbox(
             self.context, mock_container, requested_networks,
-            'kubernetes/pause')
+            requested_volumes, 'kubernetes/pause')
         self.mock_docker.create_container.assert_called_once_with(
-            'kubernetes/pause', name=sandbox_name, hostname=sandbox_name)
+            'kubernetes/pause', name=sandbox_name, hostname=sandbox_name,
+            host_config=fake_host_config, volumes=[], mac_address='fake_mac',
+            networking_config={'Id': 'val1', 'key1': 'val2'})
         self.assertEqual(result_sandbox_id, 'val1')
 
     @mock.patch('zun.network.kuryr_network.KuryrNetwork'
                 '.connect_container_to_network')
+    @mock.patch('zun.network.kuryr_network.KuryrNetwork'
+                '.create_or_update_port')
     @mock.patch('zun.container.docker.driver.DockerDriver.get_sandbox_name')
     @mock.patch('zun.common.utils.get_security_group_ids')
     def test_create_sandbox_with_long_name(self, mock_get_security_group_ids,
                                            mock_get_sandbox_name,
+                                           mock_create_or_update_port,
                                            mock_connect):
         sandbox_name = 'x' * 100
         mock_get_sandbox_name.return_value = sandbox_name
         self.mock_docker.create_container = mock.Mock(
             return_value={'Id': 'val1', 'key1': 'val2'})
+        self.mock_docker.create_networking_config = mock.Mock(
+            return_value={'Id': 'val1', 'key1': 'val2'})
+        fake_host_config = mock.Mock()
+        self.mock_docker.create_host_config.return_value = fake_host_config
         mock_container = mock.MagicMock()
-        requested_networks = []
+        mock_container.hostname = None
+        requested_networks = [{'network': 'fake-network'}]
+        requested_volumes = []
+        fake_port = {'mac_address': 'fake_mac'}
+        mock_create_or_update_port.return_value = ([], fake_port)
         result_sandbox_id = self.driver.create_sandbox(
             self.context, mock_container, requested_networks,
-            'kubernetes/pause')
+            requested_volumes, 'kubernetes/pause')
         self.mock_docker.create_container.assert_called_once_with(
-            'kubernetes/pause', name=sandbox_name, hostname=sandbox_name[:63])
+            'kubernetes/pause', name=sandbox_name, hostname=sandbox_name[:63],
+            host_config=fake_host_config, volumes=[], mac_address='fake_mac',
+            networking_config={'Id': 'val1', 'key1': 'val2'})
         self.assertEqual(result_sandbox_id, 'val1')
 
     def test_delete_sandbox(self):
@@ -465,98 +562,61 @@ class TestDockerDriver(base.DriverTestCase):
         self.assertEqual('10000000/0', stats_info['BLOCK I/O(B)'])
         self.assertEqual('200/200', stats_info['NET I/O(B)'])
 
-
-class TestNovaDockerDriver(base.DriverTestCase):
-    def setUp(self):
-        super(TestNovaDockerDriver, self).setUp()
-        self.driver = NovaDockerDriver()
-
-    @mock.patch(
-        'zun.container.docker.driver.NovaDockerDriver.get_sandbox_name')
-    @mock.patch('zun.common.nova.NovaClient')
-    @mock.patch('zun.container.docker.driver.NovaDockerDriver._ensure_active')
-    @mock.patch('zun.container.docker.driver.'
-                'NovaDockerDriver._find_container_by_server_name')
-    def test_create_sandbox(self, mock_find_container_by_server_name,
-                            mock_ensure_active, mock_nova_client,
-                            mock_get_sandbox_name):
-        nova_client_instance = mock.MagicMock()
-        nova_client_instance.create_server.return_value = 'server_instance'
-        mock_get_sandbox_name.return_value = 'test_sanbox_name'
-        mock_nova_client.return_value = nova_client_instance
-        mock_ensure_active.return_value = True
-        mock_find_container_by_server_name.return_value = \
-            'test_container_name_id'
-        mock_container = obj_utils.get_test_container(self.context,
-                                                      host=conf.CONF.host)
-        result_sandbox_id = self.driver.create_sandbox(self.context,
-                                                       mock_container)
-        mock_get_sandbox_name.assert_called_once_with(mock_container)
-        nova_client_instance.create_server.assert_called_once_with(
-            name='test_sanbox_name', image='kubernetes/pause',
-            flavor='m1.tiny', key_name=None,
-            nics='auto', availability_zone=':{0}:'.format(conf.CONF.host))
-        mock_ensure_active.assert_called_once_with(nova_client_instance,
-                                                   'server_instance')
-        mock_find_container_by_server_name.assert_called_once_with(
-            'test_sanbox_name')
-        self.assertEqual(result_sandbox_id, 'test_container_name_id')
-
-    @mock.patch('zun.common.nova.NovaClient')
-    @mock.patch('zun.container.docker.driver.'
-                'NovaDockerDriver._find_server_by_container_id')
-    @mock.patch('zun.container.docker.driver.NovaDockerDriver._ensure_deleted')
-    def test_delete_sandbox(self, mock_ensure_delete,
-                            mock_find_server_by_container_id, mock_nova_client
-                            ):
-        nova_client_instance = mock.MagicMock()
-        nova_client_instance.delete_server.return_value = 'delete_server_id'
-        mock_nova_client.return_value = nova_client_instance
-        mock_find_server_by_container_id.return_value = 'test_test_server_name'
-        mock_ensure_delete.return_value = True
-        self.driver.delete_sandbox(self.context, sandbox_id='test_sandbox_id')
-        mock_find_server_by_container_id.assert_called_once_with(
-            'test_sandbox_id')
-        nova_client_instance.delete_server.assert_called_once_with(
-            'test_test_server_name')
-        mock_ensure_delete.assert_called_once_with(nova_client_instance,
-                                                   'delete_server_id')
-
-    @mock.patch('zun.common.nova.NovaClient')
-    @mock.patch('zun.container.docker.driver.'
-                'NovaDockerDriver._find_server_by_container_id')
-    def test_stop_sandbox(self, mock_find_server_by_container_id,
-                          mock_nova_client):
-        nova_client_instance = mock.MagicMock()
-        nova_client_instance.stop_server.return_value = 'stop_server_id'
-        mock_nova_client.return_value = nova_client_instance
-        mock_find_server_by_container_id.return_value = 'test_test_server_name'
-        self.driver.stop_sandbox(self.context, sandbox_id='test_sandbox_id')
-        mock_find_server_by_container_id.assert_called_once_with(
-            'test_sandbox_id')
-        nova_client_instance.stop_server.assert_called_once_with(
-            'test_test_server_name')
-
-    @mock.patch('zun.container.docker.driver.'
-                'NovaDockerDriver._find_server_by_container_id')
-    @mock.patch('zun.common.nova.NovaClient')
-    def test_get_addresses(self, mock_nova_client,
-                           mock_find_server_by_container_id):
-        nova_client_instance = mock.MagicMock()
-        nova_client_instance.get_addresses.return_value = 'test_address'
-        mock_nova_client.return_value = nova_client_instance
-        mock_find_server_by_container_id.return_value = 'test_test_server_name'
+    @mock.patch('zun.network.kuryr_network.KuryrNetwork'
+                '.disconnect_container_from_network')
+    def test_network_detach(self, mock_detach):
         mock_container = mock.MagicMock()
-        mock_container.get_sandbox_id.return_value = 'test_sanbox_id'
-        result_address = self.driver.get_addresses(self.context,
-                                                   mock_container)
-        mock_find_server_by_container_id.assert_called_once_with(
-            'test_sanbox_id')
-        nova_client_instance.get_addresses.assert_called_once_with(
-            'test_test_server_name')
-        self.assertEqual(result_address, 'test_address')
+        self.driver.network_detach(self.context, mock_container, 'network')
+        mock_detach.assert_called_once_with(mock_container,
+                                            'network',
+                                            mock.ANY)
 
-    @mock.patch('oslo_concurrency.processutils.execute')
+    @mock.patch('zun.network.kuryr_network.KuryrNetwork'
+                '.connect_container_to_network')
+    @mock.patch('zun.network.kuryr_network.KuryrNetwork'
+                '.disconnect_container_from_network')
+    @mock.patch('zun.network.kuryr_network.KuryrNetwork'
+                '.list_networks')
+    def test_network_attach(self, mock_list, mock_disconnect, mock_connect):
+        mock_container = mock.MagicMock()
+        mock_container.security_groups = None
+        mock_list.return_value = {'network': 'network'}
+        requested_network = [{'network': 'network',
+                              'port': '',
+                              'v4-fixed-ip': '',
+                              'v6-fixed-ip': '',
+                              'preserve_on_delete': False}]
+        self.driver.network_attach(self.context, mock_container, 'network')
+        mock_connect.assert_called_once_with(mock_container,
+                                             'network',
+                                             requested_network[0],
+                                             security_groups=None)
+
+    @mock.patch('zun.common.utils.get_security_group_ids')
+    @mock.patch('zun.network.kuryr_network.KuryrNetwork'
+                '.connect_container_to_network')
+    @mock.patch('zun.network.kuryr_network.KuryrNetwork'
+                '.list_networks')
+    def test_network_attach_with_security_group(self, mock_list,
+                                                mock_connect,
+                                                mock_get_sec_group_id):
+        test_sec_group_id = '84e3a4c1-c8cd-46b1-a0d9-c8c35f6a32a4'
+        mock_container = mock.MagicMock()
+        mock_container.security_groups = ['test_sec_group']
+        mock_list.return_value = {'network': 'network'}
+        mock_get_sec_group_id.return_value = test_sec_group_id
+        requested_network = [{'network': 'network',
+                              'port': '',
+                              'v4-fixed-ip': '',
+                              'v6-fixed-ip': '',
+                              'preserve_on_delete': False}]
+        self.driver.network_attach(self.context, mock_container, 'network')
+        mock_connect.assert_called_once_with(mock_container,
+                                             'network',
+                                             requested_network[0],
+                                             security_groups=test_sec_group_id)
+
+    @mock.patch('zun.common.utils.execute')
     @mock.patch('zun.container.driver.ContainerDriver.get_host_mem')
     @mock.patch(
         'zun.container.docker.driver.DockerDriver.get_host_info')
@@ -590,52 +650,3 @@ class TestNovaDockerDriver(base.DriverTestCase):
         self.assertEqual('CentOS', node_obj.os)
         self.assertEqual('3.10.0-123', node_obj.kernel_version)
         self.assertEqual({'dev.type': 'product'}, node_obj.labels)
-
-    @mock.patch('tarfile.open')
-    def test_read_tar_image(self, mock_open):
-        fake_image = {'path': 'fake-path'}
-        mock_context_manager = mock.MagicMock()
-        mock_open.return_value = mock_context_manager
-        mock_file = mock.MagicMock()
-        mock_context_manager.__enter__.return_value = mock_file
-        mock_data = [{"Config": "fake_config",
-                      "RepoTags": ["cirros:latest"],
-                      "Layers": ["fake_layer", "fake_layer2"]}]
-        mock_file.extractfile.return_value.read.return_value = \
-            jsonutils.dumps(mock_data, separators=(',', ':'))
-
-        self.driver.read_tar_image(fake_image)
-        self.assertEqual('cirros', fake_image['repo'])
-        self.assertEqual('latest', fake_image['tag'])
-
-    @mock.patch('tarfile.open')
-    def test_read_tar_image_multi_tags(self, mock_open):
-        fake_image = {'path': 'fake-path'}
-        mock_context_manager = mock.MagicMock()
-        mock_open.return_value = mock_context_manager
-        mock_file = mock.MagicMock()
-        mock_context_manager.__enter__.return_value = mock_file
-        mock_data = [{"Config": "fake_config",
-                      "RepoTags": ["cirros:latest", "cirros:0.3.4"],
-                      "Layers": ["fake_layer", "fake_layer2"]}]
-        mock_file.extractfile.return_value.read.return_value = \
-            jsonutils.dumps(mock_data, separators=(',', ':'))
-
-        self.driver.read_tar_image(fake_image)
-        self.assertEqual('cirros', fake_image['repo'])
-        self.assertEqual('latest', fake_image['tag'])
-
-    @mock.patch('tarfile.open')
-    def test_read_tar_image_fail(self, mock_open):
-        fake_image = {'path': 'fake-path'}
-        mock_context_manager = mock.MagicMock()
-        mock_open.return_value = mock_context_manager
-        mock_file = mock.MagicMock()
-        mock_context_manager.__enter__.return_value = mock_file
-        mock_data = [{"Config": "fake_config"}]
-        mock_file.extractfile.return_value.read.return_value = \
-            jsonutils.dumps(mock_data, separators=(',', ':'))
-
-        self.driver.read_tar_image(fake_image)
-        self.assertTrue('repo' not in fake_image)
-        self.assertTrue('tag' not in fake_image)
